"use strict";
require("rxjs/add/operator/filter");
require("rxjs/add/operator/do");
require("rxjs/add/operator/take");
require("rxjs/add/operator/mergeMap");
require("rxjs/add/operator/catch");
require("rxjs/add/observable/of");
var Observable_1 = require("rxjs/Observable");
function isVisible(element, threshold, _window) {
    if (threshold === void 0) { threshold = 0; }
    if (_window === void 0) { _window = window; }
    var rect = element.getBoundingClientRect();
    var elementLargerThenViewport = rect.top <= threshold && rect.bottom >= -threshold;
    var topInsideViewport = rect.top >= 0 && rect.top <= _window.innerHeight;
    var belowInsideViewport = rect.bottom >= 0 && rect.bottom <= _window.innerHeight;
    var rightsideInViewport = rect.right >= -threshold && (rect.right - threshold) <= _window.innerWidth;
    var leftsideInViewport = rect.left >= -threshold && (rect.left - threshold) <= _window.innerWidth;
    return (elementLargerThenViewport ||
        ((topInsideViewport || belowInsideViewport) &&
            (rightsideInViewport || leftsideInViewport)));
}
function loadImage(imagePath) {
    return Observable_1.Observable
        .create(function (observer) {
        var img = new Image();
        img.src = imagePath;
        img.onload = function () {
            observer.next(imagePath);
            observer.complete();
        };
        img.onerror = function (err) {
            observer.error(null);
        };
    });
}
function setImage(element, imagePath) {
    var isImgNode = element.nodeName.toLowerCase() === 'img';
    if (isImgNode) {
        element.src = imagePath;
    }
    else {
        element.style.backgroundImage = "url('" + imagePath + "')";
    }
    return element;
}
function setLoadedStyle(element) {
    var styles = element.className
        .split(' ')
        .filter(function (s) { return !!s; })
        .filter(function (s) { return s !== 'ng2-lazyloading'; });
    styles.push('ng2-lazyloaded');
    element.className = styles.join(' ');
    return element;
}
function lazyLoadImage(image, imagePath, defaultImagePath, errorImgPath, offset) {
    if (defaultImagePath) {
        setImage(image, defaultImagePath);
    }
    return function (scrollObservable) {
        return scrollObservable
            .filter(function () { return isVisible(image, offset); })
            .take(1)
            .mergeMap(function () { return loadImage(imagePath); })
            .do(function () { return setImage(image, imagePath); })
            .catch(function () {
            if (errorImgPath) {
                setImage(image, errorImgPath);
            }
            return Observable_1.Observable.of(1);
        })
            .do(function () { return setLoadedStyle(image); });
    };
}
exports.lazyLoadImage = lazyLoadImage;
